'use strict';
/* eslint no-console: 0 */

var CaseSensitivePathsPlugin = require('case-sensitive-paths-webpack-plugin');

var colors = require('chalk');

var HtmlWebpackPlugin = require('html-webpack-plugin');

var RaxWebpackPlugin = require('rax-webpack-plugin');

var webpack = require('webpack');

var pathConfig = require('./path.config'); // Webpack uses `publicPath` to determine where the app is being served from.
// In development, we always serve from the root. This makes config easier.


var publicPath = '/'; // `publicUrl` is just like `publicPath`, but we will provide it to our app
// as %PUBLIC_URL% in `index.html` and `process.env.PUBLIC_URL` in JavaScript.
// Omit trailing slash as %PUBLIC_PATH%/xyz looks better than %PUBLIC_PATH%xyz.

var publicUrl = '';
var babelOptions = {
  presets: [require.resolve('babel-preset-es2015'), require.resolve('babel-preset-rax')],
  plugins: [require.resolve('rax-hot-loader/babel')]
};
module.exports = {
  mode: process.env.NODE_ENV,
  context: process.cwd(),
  // Compile target should "web" when use hot reload
  target: 'web',
  // devtool: 'inline-source-map',
  // These are the "entry points" to our application.
  // This means they will be the "root" imports that are included in JS bundle.
  // The first two entry points enable "hot" CSS and auto-refreshes for JS.
  entry: {},
  output: {
    // Next line is not used in dev but WebpackDevServer crashes without it:
    path: pathConfig.appBuild,
    // Add /* filename */ comments to generated require()s in the output.
    pathinfo: true,
    // This does not produce a real file. It's just the virtual path that is
    // served by WebpackDevServer in development. This is the JS bundle
    // containing code from all our entry points, and the Webpack runtime.
    filename: 'js/[name].js',
    // This is the URL that app is served from. We use "/" in development.
    publicPath: publicPath
  },
  resolve: {
    extensions: ['.js', '.json', '.jsx', '.html', '.vue']
  },
  plugins: [new RaxWebpackPlugin({
    target: 'bundle',
    externalBuiltinModules: false
  }), // Generates an `index.html` file with the <script> injected.
  new HtmlWebpackPlugin({
    inject: true,
    template: pathConfig.appHtml
  }), // Makes some environment variables available to the JS code, for example:
  // if (process.env.NODE_ENV === 'development') { ... }. See `./env.js`.
  new webpack.DefinePlugin({
    'process.env': {
      // Useful for determining whether we’re running in production mode.
      // Most importantly, it switches React into the correct mode.
      NODE_ENV: JSON.stringify(process.env.NODE_ENV || 'development'),
      // Useful for resolving the correct path to static assets in `public`.
      // For example, <img src={process.env.PUBLIC_URL + '/img/logo.png'} />.
      // This should only be used as an escape hatch. Normally you would put
      // images into the `src` and `import` them in code to get their paths.
      PUBLIC_URL: JSON.stringify(publicUrl)
    }
  }), // This is necessary to emit hot updates (currently CSS only):
  // new webpack.HotModuleReplacementPlugin(),
  // Watcher doesn't work well if you mistype casing in a path so we use
  // a plugin that prints an error when you attempt to do this.
  // See https://github.com/facebookincubator/create-react-app/issues/240
  new CaseSensitivePathsPlugin(), // show webpakc build progress
  new webpack.ProgressPlugin(function (percentage, msg) {
    var stream = process.stderr;

    if (stream.isTTY && percentage < 0.71) {
      stream.cursorTo(0);
      stream.write("webpack: ".concat(msg, "..."));
      stream.clearLine(1);
    } else if (percentage === 1) {
      console.log('');
      console.log(colors.green('webpack: bundle build is now finished.'));
    }
  })],
  module: {
    rules: [{
      test: /\.tsx?$/,
      exclude: /(node_modules|bower_components)/,
      use: [{
        loader: 'ts-loader'
      }]
    }, {
      test: /\.jsx?$/,
      exclude: /(node_modules|bower_components)/,
      use: [{
        loader: 'babel-loader',
        options: babelOptions
      }]
    }, {
      test: /\.(vue|html)$/,
      use: [{
        loader: 'sfc-loader',
        options: {
          builtInRuntime: false,
          preserveWhitespace: false,
          module: 'commonjs' // weexGlobalComponents: {
          //   button: 'rax-button'
          // },

        }
      }],
      exclude: [pathConfig.appHtml]
    }, {
      test: /\.css$/,
      use: [{
        loader: 'stylesheet-loader'
      }]
    }, // JSON is not enabled by default in Webpack but both Node and Browserify
    // allow it implicitly so we also enable it.
    {
      test: /\.json$/,
      use: [{
        loader: 'json-loader'
      }]
    }, // load inline images using image-source-loader for Image
    {
      test: /\.(png|jpe?g|gif)$/i,
      use: [{
        loader: 'image-source-loader'
      }]
    }]
  }
};